local Skada = Skada

Skada:AddLoadableModule("Damage", function(Skada, L)
	if Skada.db.profile.modulesBlocked.Damage then return end
	
	local mod = Skada:NewModule("Critical damage")
	local playerCrit = Skada:NewModule("Critical spell list")
	local spellsCrit = Skada:NewModule("Critical spell details")
	local critTarget = Skada:NewModule("Critically hit targets")

	local sourceDmg
	local function getDPS(set, player, sourceDmg)
		local totaltime = Skada:PlayerActiveTime(set, player)

		return sourceDmg / math.max(1, totaltime)
	end

	local function getRaidCritDPS(set)
		if set.time > 0 then
			return set.critDamage / math.max(1, set.time)
		else
			local endtime = set.endtime
			if not endtime then
				endtime = time()
			end
			return set.critDamage / math.max(1, endtime - set.starttime)
		end
	end

	local function log_damage(set, crit)
		-- Get the player.
		local player = Skada:get_player(set, crit.playerid, crit.playername)
		if player then

			-- Subtract overkill
	--		local amount = math.max(0,crit.amount - crit.overkill)
			-- Or don't. Seems to be the way other meters do it.
			local amount = crit.amount
			-- print(crit.playername..": "..crit.spellname.." for "..tostring(amount))

			-- Also add to set total and total non critical damage. NOT USED
			-- set.nonCritDamage = set.nonCritDamage + amount		

			-- Add spell to player if it does not exist.
			-- Add total crits if not exist
			if not player.critDamageSpells[crit.spellname] then
				player.critDamageSpells[crit.spellname] = {id = crit.spellid, totalHits = 0, damage = 0, totalCrits = 0, critDamage = 0}
			end
			
			-- Get the spell from player.
			local spell = player.critDamageSpells[crit.spellname]
			
			-- Add to player total damage.
			player.nonCritDamage = player.nonCritDamage + amount
			spell.damage = spell.damage + amount
			
			player.totalHits = player.totalHits + 1
			spell.totalHits = spell.totalHits + 1
			
			if crit.critical then
				spell.totalCrits = spell.totalCrits + 1
				spell.critDamage = spell.critDamage + amount
				
				player.totalCrits = player.totalCrits + 1
				player.critDamage = player.critDamage + amount
				
				set.critDamage = set.critDamage + amount
				
				if spell.max == nil or amount > spell.max then
					spell.max = amount
				end
				
				if (spell.min == nil or amount < spell.min) and not crit.missed then
					spell.min = amount
				end
			end
			
			-- For now, only save critDamaged info to current set.
			-- Saving this to Total may become a memory hog deluxe, and besides, it does not make much sense
			-- to see in Total. Why care which particular mob you critDamaged the most in a whole raid, for example?
			if set == Skada.current and crit.dstname and amount > 0 and crit.critical then
				-- Make sure destination exists in player.
				if not player.critDamaged[crit.dstname] then
					player.critDamaged[crit.dstname] = 0
				end

				-- Add to destination.
				player.critDamaged[crit.dstname] = player.critDamaged[crit.dstname] + amount
			end

		end
	end
		
	local crit = {}

	local function SwingDamage(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, samount, soverkill, sschool, sresisted, sblocked, sabsorbed, scritical, sglancing, scrushing, soffhand, smultistrike)
		-- White melee.
		if srcGUID ~= dstGUID then
			crit.playerid = srcGUID
			crit.playername = srcName
			crit.playerflags = srcFlags
			crit.dstname = dstName
			crit.spellid = 6603
			crit.spellname = "Attack"
			crit.amount = samount
			crit.critical = scritical
			crit.missed = nil

			Skada:FixPets(crit)
			log_damage(Skada.current, crit)
			log_damage(Skada.total, crit)
		end
	end
		
	local function SpellDamage(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, spellId, spellName, spellSchool, samount, soverkill, sschool, sresisted, sblocked, sabsorbed, scritical, sglancing, scrushing, soffhand, smultistrike)
	-- Spell damage.
		if srcGUID ~= dstGUID then
			-- XXX WoD quick fix for Mage's Prismatic Crystal talent
			-- All damage done to the crystal is transferred, so ignore it
			if dstGUID:match("^Creature%-0%-%d+%-%d+%-%d+%-76933%-%w+$") then
				return
			end
			
			-- XXX Filter spells with 100% crit chance
			-- Not worth while to include, skews everything
			-- if spellId ~= 5176 then
				-- return
			-- end
			
			crit.playerid = srcGUID
			crit.playerflags = srcFlags
			crit.dstname = dstName
			crit.playername = srcName
			crit.spellid = spellId
			crit.spellname = spellName
			crit.amount = samount
			crit.critical = scritical
			crit.missed = nil

			Skada:FixPets(crit)
			log_damage(Skada.current, crit)
			log_damage(Skada.total, crit)
		end
	end

	function mod:Update(win, set)
		-- Max value.
		local max = 0
		local nr = 1
		
		for i, player in ipairs(set.players) do
			if player.nonCritDamage > 0 then
				local CritDps = getDPS(set, player, player.critDamage)

				local d = win.dataset[nr] or {}
				win.dataset[nr] = d
				d.label = player.name
				d.value = player.critDamage
				d.id = player.id
				d.class = player.class
				d.role = player.role
				
				d.valuetext = Skada:FormatValueText(
					Skada:FormatNumber(player.critDamage), self.metadata.columns.Damage,
					Skada:FormatNumber(CritDps), self.metadata.columns.DPS,
					string.format("%02.1f%%", (player.critDamage / set.critDamage) * 100), self.metadata.columns.Percent
				)
				if player.critDamage > max then
					max = player.critDamage
				end
				nr = nr + 1
			end
		end

		win.metadata.maxvalue = max
	end
	
	local function player_tooltip(win, id, label, tooltip)
		local player = Skada:find_player(win:get_selected_set(), playerCrit.playerid)
		if player then
			local spell = player.critDamageSpells[label]
			if spell then
				tooltip:AddLine(player.name.." - "..label)
				if spell.max and spell.min then
					tooltip:AddDoubleLine(L["Minimum hit:"], Skada:FormatNumber(spell.min), 255,255,255,255,255,255)
					tooltip:AddDoubleLine(L["Maximum hit:"], Skada:FormatNumber(spell.max), 255,255,255,255,255,255)
				end
				tooltip:AddDoubleLine(L["Average hit:"], Skada:FormatNumber(spell.critDamage / spell.totalCrits), 255,255,255,255,255,255)
			end
		end
	end

	function playerCrit:Enter(win, id, label)
		local player = Skada:find_player(win:get_selected_set(), id)
		if player then
			playerCrit.playerid = id
			playerCrit.title = player.name.."'s Critical Strike Damage"
		end
	end

	function playerCrit:Update(win,set)
		-- View spells for this player.
		local player = Skada:find_player(set, self.playerid)
		local max = 0

		-- If we reset we have no data.
		if player then

			local nr = 1
			if player then
				for spellname, spell in pairs(player.critDamageSpells) do

					local d = win.dataset[nr] or {}
					win.dataset[nr] = d
					d.label = spellname
					d.id = spellname
					local _, _, icon = GetSpellInfo(spell.id)
					d.icon = icon
					d.spellid = spell.id
					d.value = spell.critDamage
					d.valuetext = Skada:FormatValueText(
						Skada:FormatNumber(spell.critDamage), self.metadata.columns.Damage,
						string.format("%02.1f%%", (spell.critDamage / player.critDamage) * 100), self.metadata.columns.Percent
					)
					if spell.critDamage > max then
						max = spell.critDamage
					end
					nr = nr + 1
				end
			end
		end

		win.metadata.maxvalue = max
	end
	
	local function add_detail_bar(win, title, value)
		local nr = spellsCrit.nr + 1
		spellsCrit.nr = nr
		local d = win.dataset[nr] or {}
		win.dataset[nr] = d

		d.value = value
		d.label = title
		d.id = title
		d.valuetext = Skada:FormatValueText(
			value, mod.metadata.columns.Damage,
			string.format("%02.1f%%", value / spellsCrit.totalHits * 100), mod.metadata.columns.Percent
		)

		win.metadata.maxvalue = math.max(win.metadata.maxvalue, value)
	end
	
	function spellsCrit:Enter(win, id, label)
		local player = Skada:find_player(win:get_selected_set(), playerCrit.playerid)
		spellsCrit.spellname = label
		spellsCrit.title = player.name.."'s "..label
	end
	
	function spellsCrit:Update(win, set)
		local player = Skada:find_player(set, playerCrit.playerid)

		if player then
		local spell = player.critDamageSpells[self.spellname]

			if spell then
				spellsCrit.totalHits = spell.totalHits
				spellsCrit.nr = 0
				win.metadata.maxvalue = 0
				
				if spell.totalHits and spell.totalHits > 0 then
					add_detail_bar(win, "Total", spell.totalHits)
				end
				if spell.totalCrits and spell.totalCrits > 0 then
					add_detail_bar(win, "Critical", spell.totalCrits)
				end
			end
		end
	end
	
	function critTarget:Enter(win, id, label)
		local player = Skada:find_player(win:get_selected_set(), id)
		critTarget.playerid = id
		critTarget.title = (player and player.name or "Unknown").."'s Critically hit targets"

	end

	function critTarget:Update(win, set)
		local player = Skada:find_player(set, self.playerid)
		local max = 0

		-- If we reset we have no data.
		if player then

			local nr = 1
			if player then
				for mob, amount in pairs(player.critDamaged) do

					local d = win.dataset[nr] or {}
					win.dataset[nr] = d
					d.label = mob
					d.id = mob
					d.value = amount
					d.valuetext = Skada:FormatValueText(
						Skada:FormatNumber(amount), self.metadata.columns.Damage,
						string.format("%02.1f%%", (amount / player.critDamage) * 100), self.metadata.columns.Percent
					)
					if amount > max then
						max = amount
					end
					nr = nr + 1
				end
			end
		end

		win.metadata.maxvalue = max
	end

	function mod:OnEnable()
		mod.metadata = {showspots = true, click1 = playerCrit, click2 = critTarget, columns = {Damage = true, DPS = true, Percent = true}}
		playerCrit.metadata	= {tooltip = player_tooltip, click1 = spellsCrit, columns = {Damage = true, Percent = true}}
		spellsCrit.metadata = {columns = {Damage = true, Percent = true}}
		critTarget.metadata = {showspots = true, columns = {Damage = true, Percent = true}}
			

		Skada:RegisterForCL(SwingDamage, 'SWING_DAMAGE', {src_is_interesting = true, dst_is_not_interesting = true})
		
		Skada:RegisterForCL(SpellDamage, 'DAMAGE_SHIELD', {src_is_interesting = true, dst_is_not_interesting = true})
		Skada:RegisterForCL(SpellDamage, 'SPELL_DAMAGE', {src_is_interesting = true, dst_is_not_interesting = true})
		Skada:RegisterForCL(SpellDamage, 'SPELL_PERIODIC_DAMAGE', {src_is_interesting = true, dst_is_not_interesting = true})
		Skada:RegisterForCL(SpellDamage, 'SPELL_BUILDING_DAMAGE', {src_is_interesting = true, dst_is_not_interesting = true})
		Skada:RegisterForCL(SpellDamage, 'RANGE_DAMAGE', {src_is_interesting = true, dst_is_not_interesting = true})
		
		Skada:AddMode(self)
	end

	function mod:OnDisable()
		Skada:RemoveMode(self)
	end

	function mod:AddPlayerAttributes(player)
		if not player.critDamage or not player.critDamageSpells or not player.totalCrits or not player.totalHits or not player.nonCritDamage then
			player.critDamage = player.critDamage or 0					-- Total crit damage
			player.critDamageSpells = player.critDamageSpells or {}		-- Stored spell crit info
			player.totalCrits = player.totalCrits or 0
			player.totalHits = player.totalHits or 0
			player.nonCritDamage = player.nonCritDamage or 0
		end
		
		if not player.critDamaged then
			player.critDamaged = {}
		end
	end

	function mod:AddToTooltip(set, tooltip)
		GameTooltip:AddDoubleLine("Critical Strike DPS", Skada:FormatNumber(getRaidCritDPS(set)), 1,1,1)
	end

	function mod:AddSetAttributes(set)
		if not set.nonCritDamage then
			set.nonCritDamage = 0
		end
		if not set.critDamage then
			set.critDamage = 0
		end
	end

	function mod:GetSetSummary(set)
		return Skada:FormatValueText(
			Skada:FormatNumber(set.critDamage), self.metadata.columns.Damage,
			Skada:FormatNumber(getRaidCritDPS(set)), self.metadata.columns.DPS
		)
	end
end)