local Skada = Skada

Skada:AddLoadableModule("Healing", function(Skada, L)
	if Skada.db.profile.modulesBlocked.Healing then return end
	
	local mod = Skada:NewModule("Critical healing")
	local playerCrit = Skada:NewModule("Critical healing spell list")
	local spellsCrit = Skada:NewModule("Critical healing spell details")
	local critTarget = Skada:NewModule("Critical healing targets")

	local sourceHeal
	local function getCritHPS(set, player, sourceHeal)
		local totaltime = Skada:PlayerActiveTime(set, player)

		return sourceHeal / math.max(1, totaltime)
	end

	local function getRaidCritHPS(set)
		if set.time > 0 then
			return set.critHealing / math.max(1, set.time)
		else
			local endtime = set.endtime
			if not endtime then
				endtime = time()
			end
			return set.critHealing / math.max(1, endtime - set.starttime)
		end
	end

	local function log_healing(set, crit)
		-- Get the player.
		local player = Skada:get_player(set, crit.playerid, crit.playername)
		if player then

			-- Subtract overkill
	--		local amount = math.max(0,crit.amount - crit.overkill)
			-- Or don't. Seems to be the way other meters do it.
			local amount = crit.amount
			-- print(crit.playername..": "..crit.spellname.." for "..tostring(amount))

			-- Also add to set total and total non critical healing. NOT USED
			-- set.nonCritHealing = set.nonCritHealing + amount		

			-- Add spell to player if it does not exist.
			-- Add total crits if not exist
			if not player.critHealingSpells[crit.spellname] then
				player.critHealingSpells[crit.spellname] = {id = crit.spellid, totalHitHeals = 0, healing = 0, totalCritHeals = 0, critHealing = 0}
			end
			
			-- Get the spell from player.
			local spell = player.critHealingSpells[crit.spellname]
			
			-- Add to player total healing.
			player.nonCritHealing = player.nonCritHealing + amount
			spell.healing = spell.healing + amount
			
			player.totalHitHeals = player.totalHitHeals + 1
			spell.totalHitHeals = spell.totalHitHeals + 1
			
			if crit.critical then
				spell.totalCritHeals = spell.totalCritHeals + 1
				spell.critHealing = spell.critHealing + amount
				
				player.totalCritHeals = player.totalCritHeals + 1
				player.critHealing = player.critHealing + amount
				
				set.critHealing = set.critHealing + amount
				
				if spell.max == nil or amount > spell.max then
					spell.max = amount
				end
				
				if (spell.min == nil or amount < spell.min) then
					spell.min = amount
				end
				
				do
					if crit.dstName and crit.dstGUID then
						local critHealed = player.critHealed[crit.dstGUID]

						-- Create recipient if it does not exist.
						if not critHealed then
							local _, className = UnitClass(crit.dstName)
							local playerRole = UnitGroupRolesAssigned(crit.dstName)
							critHealed = {name = crit.dstName, class = className, role = playerRole, amount = 0}
							player.critHealed[crit.dstGUID] = critHealed
						end

						critHealed.amount = critHealed.amount + amount
					end
				end
			end
		end
	end
		
	local crit = {}
		
	local function SpellHeal(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...)
		-- Healing
		local spellId, spellName, spellSchool, samount, soverhealing, absorbed, scritical = ...	-- Spell healing.
		
		if bit.band(srcFlags, COMBATLOG_OBJECT_CONTROL_PLAYER) ~= 0 and bit.band(srcFlags, dstFlags, COMBATLOG_OBJECT_REACTION_MASK) ~= 0 then
			crit.dstName = dstName
			crit.dstGUID = dstGUID
			crit.playerid = srcGUID
			crit.playername = srcName
			crit.spellid = spellId
			crit.spellname = spellName
			crit.amount = samount
			crit.critical = scritical

			Skada:FixPets(crit)
			log_healing(Skada.current, crit)
			log_healing(Skada.total, crit)
		end
	end

	function mod:Update(win, set)
		-- Max value.
		local max = 0
		local nr = 1
		
		for i, player in ipairs(set.players) do
			if player.critHealing > 0 then
				local CritHPS = getCritHPS(set, player, player.critHealing)

				local d = win.dataset[nr] or {}
				win.dataset[nr] = d
				d.label = player.name
				d.value = player.critHealing
				d.id = player.id
				d.class = player.class
				d.role = player.role
				
				d.valuetext = Skada:FormatValueText(
					Skada:FormatNumber(player.critHealing), self.metadata.columns.Healing,
					Skada:FormatNumber(CritHPS), self.metadata.columns.HPS,
					string.format("%02.1f%%", (player.critHealing / set.critHealing) * 100), self.metadata.columns.Percent
				)
				if player.critHealing > max then
					max = player.critHealing
				end
				nr = nr + 1
			end
		end

		win.metadata.maxvalue = max
	end
	
	local function player_tooltip(win, id, label, tooltip)
		local player = Skada:find_player(win:get_selected_set(), playerCrit.playerid)
		if player then
			local spell = player.critHealingSpells[label]
			if spell then
				tooltip:AddLine(player.name.." - "..label)
				if spell.max and spell.min then
					tooltip:AddDoubleLine(L["Minimum hit:"], Skada:FormatNumber(spell.min), 255,255,255,255,255,255)
					tooltip:AddDoubleLine(L["Maximum hit:"], Skada:FormatNumber(spell.max), 255,255,255,255,255,255)
				end
				tooltip:AddDoubleLine(L["Average hit:"], Skada:FormatNumber(spell.critHealing / spell.totalCritHeals), 255,255,255,255,255,255)
			end
		end
	end

	function playerCrit:Enter(win, id, label)
		local player = Skada:find_player(win:get_selected_set(), id)
		if player then
			playerCrit.playerid = id
			playerCrit.title = player.name.."'s Critical Strike Healing"
		end
	end

	function playerCrit:Update(win,set)
		-- View spells for this player.
		local player = Skada:find_player(set, self.playerid)
		local max = 0

		-- If we reset we have no data.
		if player then

			local nr = 1
			if player then
				for spellname, spell in pairs(player.critHealingSpells) do

					local d = win.dataset[nr] or {}
					win.dataset[nr] = d
					d.label = spellname
					d.id = spellname
					local _, _, icon = GetSpellInfo(spell.id)
					d.icon = icon
					d.spellid = spell.id
					d.value = spell.critHealing
					d.valuetext = Skada:FormatValueText(
						Skada:FormatNumber(spell.critHealing), self.metadata.columns.Healing,
						string.format("%02.1f%%", (spell.critHealing / player.critHealing) * 100), self.metadata.columns.Percent
					)
					if spell.critHealing > max then
						max = spell.critHealing
					end
					nr = nr + 1
				end
			end
		end

		win.metadata.maxvalue = max
	end
	
	local function add_detail_bar(win, title, value)
		local nr = spellsCrit.nr + 1
		spellsCrit.nr = nr
		local d = win.dataset[nr] or {}
		win.dataset[nr] = d

		d.value = value
		d.label = title
		d.id = title
		d.valuetext = Skada:FormatValueText(
			value, mod.metadata.columns.Healing,
			string.format("%02.1f%%", value / spellsCrit.totalHitHeals * 100), mod.metadata.columns.Percent
		)

		win.metadata.maxvalue = math.max(win.metadata.maxvalue, value)
	end
	
	function spellsCrit:Enter(win, id, label)
		local player = Skada:find_player(win:get_selected_set(), playerCrit.playerid)
		spellsCrit.spellname = label
		spellsCrit.title = player.name.."'s "..label
	end
	
	function spellsCrit:Update(win, set)
		local player = Skada:find_player(set, playerCrit.playerid)

		if player then
		local spell = player.critHealingSpells[self.spellname]

			if spell then
				spellsCrit.totalHitHeals = spell.totalHitHeals
				spellsCrit.nr = 0
				win.metadata.maxvalue = 0
				
				if spell.totalHitHeals and spell.totalHitHeals > 0 then
					add_detail_bar(win, "Total", spell.totalHitHeals)
				end
				if spell.totalCritHeals and spell.totalCritHeals > 0 then
					add_detail_bar(win, "Critical", spell.totalCritHeals)
				end
			end
		end
	end
	
	function critTarget:Enter(win, id, label)
		local player = Skada:find_player(win:get_selected_set(), id)
		critTarget.playerid = id
		critTarget.title = (player and player.name or "Unknown").."'s Critically healed targets"

	end

	function critTarget:Update(win, set)
		local player = Skada:find_player(set, self.playerid)
		local max = 0
		local nr = 1
		
		if player then
			for dstGUID, critHealed in pairs(player.critHealed) do
				if critHealed.amount > 0 then

					local d = win.dataset[nr] or {}
					win.dataset[nr] = d

					d.id = dstGUID
					d.label = critHealed.name or dstGUID
					d.value = critHealed.amount
					d.class = critHealed.class
                    d.role = player.role
					d.valuetext = Skada:FormatValueText(
						Skada:FormatNumber(critHealed.amount), self.metadata.columns.Healing,
						string.format("%02.1f%%", critHealed.amount / player.critHealing * 100), self.metadata.columns.Percent
						)
					if critHealed.amount > max then
						max = critHealed.amount
					end

					nr = nr + 1
				end
			end
		end

		win.metadata.maxvalue = max
		win.metadata.hasicon = true
	end

	function mod:OnEnable()
		mod.metadata = {showspots = true, click1 = playerCrit, click2 = critTarget, columns = {Healing = true, HPS = true, Percent = true}}
		playerCrit.metadata	= {tooltip = player_tooltip, click1 = spellsCrit, columns = {Healing = true, Percent = true}}
		spellsCrit.metadata = {columns = {Healing = true, Percent = true}}
		critTarget.metadata = {showspots = true, columns = {Healing = true, Percent = true}}
			

		Skada:RegisterForCL(SpellHeal, 'SPELL_HEAL', {src_is_interesting = true})
		Skada:RegisterForCL(SpellHeal, 'SPELL_PERIODIC_HEAL', {src_is_interesting = true})
		
		Skada:AddMode(self)
	end

	function mod:OnDisable()
		Skada:RemoveMode(self)
	end

	function mod:AddPlayerAttributes(player)
		if not player.critHealing or not player.critHealingSpells or not player.totalCritHeals or not player.totalHitHeals or not player.nonCritHealing then
			player.critHealing = player.critHealing or 0					-- Total crit healing
			player.critHealingSpells = player.critHealingSpells or {}		-- Stored spell crit info
			player.totalCritHeals = player.totalCritHeals or 0
			player.totalHitHeals = player.totalHitHeals or 0
			player.nonCritHealing = player.nonCritHealing or 0
		end
		
		if not player.critHealed then
			player.critHealed = {}
		end
	end

	function mod:AddToTooltip(set, tooltip)
		GameTooltip:AddDoubleLine("Critical Strike HPS", Skada:FormatNumber(getRaidCritHPS(set)), 1,1,1)
	end

	function mod:AddSetAttributes(set)
		if not set.nonCritHealing then
			set.nonCritHealing = 0
		end
		if not set.critHealing then
			set.critHealing = 0
		end
	end

	function mod:GetSetSummary(set)
		return Skada:FormatValueText(
			Skada:FormatNumber(set.critHealing), self.metadata.columns.Healing,
			Skada:FormatNumber(getRaidCritHPS(set)), self.metadata.columns.HPS
		)
	end
end)