local Skada = Skada

Skada:AddLoadableModule("Healing", function(Skada, L)
	if Skada.db.profile.modulesBlocked.Healing then return end
	
	local mod = Skada:NewModule("Demon Hunter Souls Consumed")
	local soulsHealingDetails = Skada:NewModule("Demon Hunter Souls Healing Info")
	
	local sourceHeal
	local function getHPS(set, player, sourceHeal)
		local totaltime = Skada:PlayerActiveTime(set, player)

		return sourceHeal / math.max(1, totaltime)
	end

	local function log_souls(set, souls)
		local player = Skada:get_player(set, souls.playerid, souls.playername)
		
		if player then
			local actualHealing = math.max(0, souls.amount - souls.overheal)
			
			if not player then
				player = {totalHealing = 0, effectiveHealing = 0, overhealing = 0, count = 0}
			end
			
			player.totalHealing = player.totalHealing + souls.amount
			player.effectiveHealing = player.effectiveHealing + actualHealing
			player.overhealing = player.overhealing + souls.overheal
			player.count = player.count + 1
			
			set.effectiveHealing = set.effectiveHealing + actualHealing
		end
	end
		
	local souls = {}
		
	local function SpellHeal(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...)
		local spellId, spellName, spellSchool, samount, soverhealing, absorbed, scritical = ...
		
		if spellId == 203794 or spellId == 228532 then
			souls.playerid = srcGUID
			souls.playerflags = srcFlags
			souls.playername = srcName
			souls.amount = samount
			souls.overheal = soverhealing
			
			log_souls(Skada.current, souls)
			log_souls(Skada.total, souls)
		end
	end

	function mod:Update(win, set)
		local max = 0
		local nr = 1
		
		for i, player in ipairs(set.players) do
			if player.count > 0 then
				local d = win.dataset[nr] or {}
				win.dataset[nr] = d
				d.label = player.name

				d.value = player.effectiveHealing
				d.id = player.id
				d.class = player.class
				d.role = player.role
				
				local effectiveHealing
				if set.effectiveHealing == 0 then
					effectiveHealing = 1
				else
					effectiveHealing = set.effectiveHealing
				end
				
				d.valuetext = Skada:FormatValueText(
					Skada:FormatNumber(player.effectiveHealing), self.metadata.columns.Healing,
					Skada:FormatNumber(getHPS(set, player, player.effectiveHealing)), self.metadata.columns.HPS,
					string.format("%02.1f%%", player.effectiveHealing / effectiveHealing * 100), self.metadata.columns.Percent
				)
				
				if player.effectiveHealing > max then
					max = player.effectiveHealing
				end
				nr = nr + 1
			end
		end

		win.metadata.maxvalue = max
	end
	
	local function add_detail_bar(win, title, value, limit, player, set)
		local nr = soulsHealingDetails.nr + 1
		soulsHealingDetails.nr = nr
		local d = win.dataset[nr] or {}
		win.dataset[nr] = d

		d.value = value
		d.label = title
		d.id = title
		if limit then
			d.valuetext = Skada:FormatValueText(
				Skada:FormatNumber(value), soulsHealingDetails.metadata.columns.Healing,
				Skada:FormatNumber(getHPS(set, player, value)), soulsHealingDetails.metadata.columns.HPS
			)
		else
			d.valuetext = tostring(value)
		end

	end
	
	function soulsHealingDetails:Enter(win, id, label)
		local player = Skada:find_player(win:get_selected_set(), id)
		if player then
			soulsHealingDetails.playerid = id
			soulsHealingDetails.title = player.name.."'s Soul details"
		end
	end
	--
	function soulsHealingDetails:Update(win, set)
		local player = Skada:find_player(set, soulsHealingDetails.playerid)

		if player then
			soulsHealingDetails.totalRTBCasts = player.totalRTBCasts
			soulsHealingDetails.nr = 0
			win.metadata.maxvalue = 0
			
			if player.effectiveHealing and player.effectiveHealing > 0 then
				add_detail_bar(win, "Healing", player.effectiveHealing, player.totalHealing, player, set)
			end
			if player.count and player.count > 0 then
				add_detail_bar(win, "Souls Consumed", player.count)
			end
		end
	end

	function mod:OnEnable()
		mod.metadata = {showspots = true, click1 = soulsHealingDetails, columns = {Healing = true, HPS = true, Percent = true}}
		soulsHealingDetails.metadata = {columns = {Healing = true, HPS = true}}
			
		Skada:RegisterForCL(SpellHeal, 'SPELL_HEAL', {src_is_interesting = true})
		Skada:RegisterForCL(SpellHeal, 'SPELL_PERIODIC_HEAL', {src_is_interesting = true})
		
		Skada:AddMode(self)
	end

	function mod:OnDisable()
		Skada:RemoveMode(self)
	end

	function mod:AddPlayerAttributes(player)
		if not player.totalHealing or not player.effectiveHealing or not player.overhealing or not player.count then
			player.totalHealing = player.totalHealing or 0
			player.effectiveHealing = player.effectiveHealing or 0
			player.overhealing = player.overhealing or 0
			player.count = player.count or 0
		end
	end

	function mod:AddSetAttributes(set)
		if not set.effectiveHealing then
			set.effectiveHealing = 0
		end
	end
end)