local Skada = Skada
RTB_LGIST = LibStub:GetLibrary("LibGroupInSpecT-1.1")

local RTBBuffs = {
	[GetSpellInfo(199603)] = 199603, -- Jolly Roger
	[GetSpellInfo(193358)] = 193358, -- Grand Melee
	[GetSpellInfo(193357)] = 193357, -- Shark Infested Waters
	[GetSpellInfo(193359)] = 193359, -- True Bearing
	[GetSpellInfo(199600)] = 199600, -- Buried Treasure
	[GetSpellInfo(193356)] = 193356, -- Broadsides
}

Skada:AddLoadableModule("Damage", function(Skada, L)
	if Skada.db.profile.modulesBlocked.Damage then return end
	
	local mod = Skada:NewModule("Roll The Bones")
	local playerRTB = Skada:NewModule("Roll The Bones buff list")
	local spellsRTB = Skada:NewModule("Roll The Bones buff details")

	local function log_buff(set, rtb)
		local player = Skada:get_player(set, rtb.playerid, rtb.playername)
		
		if player then
			for i = 1, #rtb.RTBBuffs do
				local SpellID = rtb.RTBBuffs[i].spellID
				local SpellName = rtb.RTBBuffs[i].spellName
				
				if not player.rtb[SpellName] then
					player.rtb[SpellName] = {spellID = rtb.RTBBuffs[i].spellID, casts = 0}
				end
				
				if not set.spellName[SpellName] then
					set.spellName[SpellName] = 0
				end
				
				if not player.totalRTBCasts then
					player.totalRTBCasts = 0
				end
				
				player.rtb[SpellName].casts = player.rtb[SpellName].casts + 1
				
				set.spellName[SpellName] = set.spellName[SpellName] + 1
			end
			
			player.totalRTBCasts = player.totalRTBCasts + 1
			set.totalRTBCasts = set.totalRTBCasts + 1
		end
	end
		
	local rtb = {}
		
	local function SpellCast(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, spellId, spellName, spellSchool)
		if spellId == 193316 then
			rtb = {}
			
			rtb.playerid = srcGUID
			rtb.playerflags = srcFlags
			rtb.playername = srcName
			
			rtb.RTBBuffs = {}
			
			for i = 1, 40 do
				local spellName, _, _, _, _, duration, expiration, _, _, _, spellID, _, _, selfCast = UnitAura(RTB_LGIST:GuidToUnit(srcGUID), i)
				
				if selfCast and RTBBuffs[spellName] then
					if (expiration - GetTime()) > (duration - 0.5) then
						table.insert(rtb.RTBBuffs, {spellName = spellName, spellID = spellID})
					end
				end
			end
			
			log_buff(Skada.current, rtb)
			log_buff(Skada.total, rtb)
		end
	end

	function mod:Update(win, set)
		local max = 0
		local nr = 1
		
		for i, player in ipairs(set.players) do
			if player.totalRTBCasts > 0 then
				local d = win.dataset[nr] or {}
				win.dataset[nr] = d
				d.label = player.name

				d.value = player.totalRTBCasts
				d.id = player.id
				d.class = player.class
				d.role = player.role
				
				d.valuetext = tostring(player.totalRTBCasts)
				
				if player.totalRTBCasts > max then
					max = player.totalRTBCasts
				end
				nr = nr + 1
			end
		end

		win.metadata.maxvalue = max
	end
	
	function playerRTB:Enter(win, id, label)
		local player = Skada:find_player(win:get_selected_set(), id)
		if player then
			playerRTB.playerid = id
			playerRTB.title = player.name.."'s Roll the bones buffs"
		end
	end

	function playerRTB:Update(win,set)
		-- View spells for this player.
		local player = Skada:find_player(set, self.playerid)
		local max = 0

		-- If we reset we have no data.
		if player then

			local nr = 1
			if player then
				for spellName, spell in pairs(player.rtb) do

					local d = win.dataset[nr] or {}
					win.dataset[nr] = d
					d.label = spellName
					d.id = spellName
					local _, _, icon = GetSpellInfo(spell.spellID)
					d.icon = icon
					d.spellid = spell.spellID
					d.value = spell.casts
					d.valuetext = Skada:FormatValueText(
						spell.casts, self.metadata.columns.Damage,
						string.format("%02.1f%%", (spell.casts / player.totalRTBCasts) * 100), self.metadata.columns.Percent
					)
					if spell.casts > max then
						max = spell.casts
					end
					nr = nr + 1
				end
			end
		end

		win.metadata.maxvalue = max
	end
	
	local function add_detail_bar(win, title, value)
		local nr = spellsRTB.nr + 1
		spellsRTB.nr = nr
		local d = win.dataset[nr] or {}
		win.dataset[nr] = d

		d.value = value
		d.label = title
		d.id = title
		d.valuetext = Skada:FormatValueText(
			value, mod.metadata.columns.Damage,
			string.format("%02.1f%%", value / spellsRTB.totalRTBCasts * 100), mod.metadata.columns.Percent
		)

		win.metadata.maxvalue = math.max(win.metadata.maxvalue, value)
	end
	
	function spellsRTB:Enter(win, id, label)
		local player = Skada:find_player(win:get_selected_set(), playerRTB.playerid)
		spellsRTB.spellName = label
		spellsRTB.title = player.name.."'s "..label
	end
	--
	function spellsRTB:Update(win, set)
		local player = Skada:find_player(set, playerRTB.playerid)

		if player then
		local spell = player.rtb[self.spellName]

			if spell then
				spellsRTB.totalRTBCasts = player.totalRTBCasts
				spellsRTB.nr = 0
				win.metadata.maxvalue = 0
				
				if player.totalRTBCasts and player.totalRTBCasts > 0 then
					add_detail_bar(win, "Total RTB Casts", player.totalRTBCasts)
				end
				if spell.casts and spell.casts > 0 then
					add_detail_bar(win, "Times Buffed", spell.casts)
				end
			end
		end
	end

	function mod:OnEnable()
		mod.metadata = {showspots = true, click1 = playerRTB, columns = {Damage = true, Percent = true}}
		playerRTB.metadata	= {tooltip = player_tooltip, click1 = spellsRTB, columns = {Damage = true, Percent = true}}
		spellsRTB.metadata = {columns = {Damage = true, Percent = true}}
			
		Skada:RegisterForCL(SpellCast, 'SPELL_CAST_SUCCESS', {src_is_interesting = true, dst_is_not_interesting = true})
		
		Skada:AddMode(self)
	end

	function mod:OnDisable()
		Skada:RemoveMode(self)
	end

	function mod:AddPlayerAttributes(player)
		if not player.rtb or not player.totalRTBCasts then
			player.rtb = player.rtb or {}
			player.totalRTBCasts = player.totalRTBCasts or 0
		end
	end

	function mod:AddSetAttributes(set)
		if not set.totalRTBCasts or not set.spellName then
			set.spellName = set.spellName or {}
			set.totalRTBCasts = 0
		end
	end
end)